"""
xwnode Wrapper Package

This is a convenience wrapper that provides 'import xwnode' functionality.
The actual implementation is in the exonware-xwnode package.

Company: eXonware.com
Author: Eng. Muhammad AlShehri
Email: connect@exonware.com
"""

# This wrapper package depends on exonware-xwnode
# When you pip install xwnode, it automatically installs exonware-xwnode
# Both import paths then work:
#   - import xwnode
#   - import exonware.xwnode

__all__ = []
